/*
 * This header provides constants for pca9532 LED bindings.
 *
 * This file is licensed under the terms of the GNU General Public
 * License version 2.  This program is licensed "as is" without any
 * warranty of any kind, whether express or implied.
 */

#ifndef _DT_BINDINGS_LEDS_PCA9532_H
#define _DT_BINDINGS_LEDS_PCA9532_H

#define PCA9532_TYPE_NONE         0
#define PCA9532_TYPE_LED          1
#define PCA9532_TYPE_N2100_BEEP   2
#define PCA9532_TYPE_GPIO         3
#define PCA9532_LED_TIMER2        4

#endif /* _DT_BINDINGS_LEDS_PCA9532_H */
